@@ -399,28 +399,164 @@ def pipeline(
     strategy : str, optional
         DEPRECATED. This parameter is no longer used and will be removed in a future version.
     """
+    import sys
+    from datetime import datetime
+
+    from credtools.utils import create_float_format_dict
+
     if not os.path.exists(outdir):
         os.makedirs(outdir)
-    locus_set = load_locus_set(loci_df, calculate_lambda_s=calculate_lambda_s)
-    # meta-analysis
-    locus_set = meta(locus_set, meta_method=meta_method)
-    logger.info(f"Meta-analysis complete, {locus_set.n_loci} loci loaded.")
-    logger.info(f"Save meta-analysis results to {outdir}.")
-    for locus in locus_set.loci:
-        out_prefix = f"{outdir}/{locus.prefix}"
-        locus.sumstats.to_csv(f"{out_prefix}.sumstat", sep="\t", index=False)
-        np.savez_compressed(f"{out_prefix}.ld.npz", ld=locus.ld.r.astype(np.float16))
-        locus.ld.map.to_csv(f"{out_prefix}.ldmap", sep="\t", index=False)
-    # QC
-    if not skip_qc:
-        qc_metrics = locus_qc(locus_set)
-        logger.info(f"QC complete, {qc_metrics.keys()} metrics saved.")
-        for k, v in qc_metrics.items():
-            v.to_csv(f"{outdir}/{k}.txt", sep="\t", index=False, float_format="%.6f")
-    # fine-mapping
-    creds = fine_map(locus_set, tool=tool, strategy=strategy, **kwargs)
-    creds.pips.to_csv(f"{outdir}/pips.txt", sep="\t", header=False, index=True)
-    with open(f"{outdir}/creds.json", "w") as f:
-        json.dump(creds.to_dict(), f, indent=4)
-    logger.info(f"Fine-mapping complete, {creds.n_cs} credible sets saved.")
+
+    # Initialize run summary
+    run_summary = {
+        "start_time": datetime.now().isoformat(),
+        "total_loci": 0,
+        "successful_loci": 0,
+        "failed_loci": 0,
+        "errors": [],
+        "tool": tool,
+        "meta_method": meta_method,
+        "parameters": kwargs,
+    }
+
+    # Collect all credible sets for summary
+    all_credible_sets = []
+
+    try:
+        locus_set = load_locus_set(loci_df, calculate_lambda_s=calculate_lambda_s)
+        run_summary["total_loci"] = locus_set.n_loci
+
+        # meta-analysis
+        locus_set = meta(locus_set, meta_method=meta_method)
+        logger.info(f"Meta-analysis complete, {locus_set.n_loci} loci loaded.")
+        logger.info(f"Save meta-analysis results to {outdir}.")
+
+        for locus in locus_set.loci:
+            out_prefix = f"{outdir}/{locus.prefix}"
+            locus.sumstats.to_csv(f"{out_prefix}.sumstat", sep="\t", index=False)
+            np.savez_compressed(
+                f"{out_prefix}.ld.npz", ld=locus.ld.r.astype(np.float16)
+            )
+            locus.ld.map.to_csv(f"{out_prefix}.ldmap", sep="\t", index=False)
+
+        # QC
+        if not skip_qc:
+            qc_metrics = locus_qc(locus_set)
+            logger.info(f"QC complete, {qc_metrics.keys()} metrics saved.")
+            for k, v in qc_metrics.items():
+                v.to_csv(
+                    f"{outdir}/{k}.txt", sep="\t", index=False, float_format="%.6f"
+                )
+
+        # fine-mapping
+        try:
+            creds = fine_map(locus_set, tool=tool, strategy=strategy, **kwargs)
+            run_summary["successful_loci"] = locus_set.n_loci
+
+            # Create enhanced PIPs DataFrame
+            enhanced_pips = creds.create_enhanced_pips_df(locus_set)
+
+            # Get appropriate float formats for columns
+            format_dict = create_float_format_dict(enhanced_pips)
+
+            # Save enhanced PIPs with compression
+            output_file = f"{outdir}/pips.txt.gz"
+            enhanced_pips.to_csv(
+                output_file,
+                sep="\t",
+                index=False,
+                compression="gzip",
+                float_format=lambda x: "%.4f" % x,  # Default format
+            )
+
+            # Apply specific formats per column if needed
+            # Note: pandas doesn't support per-column float_format directly in to_csv
+            # So we'll format the DataFrame first
+            for col, fmt in format_dict.items():
+                if col in enhanced_pips.columns:
+                    if fmt == "%.3e":
+                        enhanced_pips[col] = enhanced_pips[col].apply(
+                            lambda x: f"{x:.3e}" if pd.notna(x) else ""
+                        )
+                    elif fmt == "%.4f":
+                        enhanced_pips[col] = enhanced_pips[col].apply(
+                            lambda x: f"{x:.4f}" if pd.notna(x) else ""
+                        )
+
+            # Save with formatted values
+            enhanced_pips.to_csv(
+                output_file,
+                sep="\t",
+                index=False,
+                compression="gzip",
+            )
+
+            # Create credible sets summary (all SNPs with CRED != 0)
+            cs_summary = enhanced_pips[enhanced_pips["CRED"] != 0].copy()
+            if len(cs_summary) > 0:
+                cs_summary["locus_id"] = (
+                    f"{locus_set.chrom}_{locus_set.start}_{locus_set.end}"
+                )
+                cs_summary.to_csv(
+                    f"{outdir}/credible_sets_summary.txt.gz",
+                    sep="\t",
+                    index=False,
+                    compression="gzip",
+                )
+                all_credible_sets.append(cs_summary)
+
+            # Save parameters (without lead_snps, snps, cs_sizes)
+            parameters_dict = {
+                "tool": creds.tool,
+                "n_cs": creds.n_cs,
+                "coverage": creds.coverage,
+                "parameters": creds.parameters,
+            }
+            with open(f"{outdir}/parameters.json", "w") as f:
+                json.dump(parameters_dict, f, indent=4)
+
+            logger.info(f"Fine-mapping complete, {creds.n_cs} credible sets saved.")
+
+        except Exception as e:
+            error_msg = f"Fine-mapping failed: {str(e)}"
+            logger.error(error_msg)
+            print(f"ERROR: {error_msg}", file=sys.stderr)
+            run_summary["failed_loci"] = locus_set.n_loci
+            run_summary["errors"].append(error_msg)
+
+    except Exception as e:
+        error_msg = f"Pipeline failed: {str(e)}"
+        logger.error(error_msg)
+        print(f"ERROR: {error_msg}", file=sys.stderr)
+        run_summary["errors"].append(error_msg)
+
+    finally:
+        # Generate run summary
+        run_summary["end_time"] = datetime.now().isoformat()
+        _generate_run_summary(run_summary, f"{outdir}/run_summary.log")
+
     return
+
+
+def _generate_run_summary(run_summary: dict, output_file: str):
+    """Generate run summary log file."""
+    with open(output_file, "w") as f:
+        f.write("=== CREDTOOLS FINE-MAPPING RUN SUMMARY ===\n")
+        f.write(f"Start Time: {run_summary['start_time']}\n")
+        f.write(f"End Time: {run_summary['end_time']}\n")
+        f.write(f"Total Loci: {run_summary['total_loci']}\n")
+        f.write(f"Successful: {run_summary['successful_loci']}\n")
+        f.write(f"Failed: {run_summary['failed_loci']}\n")
+        f.write("\n")
+
+        if run_summary["errors"]:
+            f.write("Error Details:\n")
+            for error in run_summary["errors"]:
+                f.write(f"  - {error}\n")
+            f.write("\n")
+
+        f.write("Parameters Used:\n")
+        f.write(f"  Tool: {run_summary['tool']}\n")
+        f.write(f"  Meta Method: {run_summary['meta_method']}\n")
+        for key, value in run_summary["parameters"].items():
+            f.write(f"  {key}: {value}\n")
